%%   demo_Sparse_Phase_Amplitude_Reconstruction.m
% 
%%   Demo for the paper:
%%   V. Katkovnik, J. Astola
%%   "High-resolution wave field reconstruction: decoupled inverse imaging with sparse regularization 
%%   for phase and amplitude", (03/16/2011), Submitted to JASA A

%% Select one of the object models: lines 29-30, 34-35, 38-39
%% Select the algorithm's parameters: lines 70 or 75

%%  Katkovnik V., August  2011
close all
clear all
'START'
tic

'Start: demo_Sparse_Phase_Ampl_Reconstruction.m'


%%%%%%%%%% TEST-IMAGES %%%%%%%%%%%%%%%%%%%%%

% y=im2double(imread('image_Lena256.png'));
% y=im2double(imread('image_Lena512.png'));
% y=im2double(imread('peppers512.png'));
% y=im2double(imread('Cameraman256.png'))*0+.1;
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %% SELECT ONE OF THE OBJECT MODELS
 %% PHASE OBJECT %%%%%%%%%%%%%%%%%%%%%%%%%%%
   y_phi = checkerboard(64,2,2); y_abs=y_phi; y_abs(:)=1;
 %  y_phi=im2double(imread('Cameraman256.png'));  y_abs=y_phi; y_abs(:)=1;
%   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
   %% AMPLITUDE OBJECT %%%%%%%%%%%%%%%%%%%%%%%%%%%
%  y_abs = checkerboard(64,2,2)+.2;; y_phi=y_abs; y_phi(:)=1;
%  y_abs=im2double(imread('Cameraman256.png'))+.2; y_phi=y_abs;  y_phi(:)=1;
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   %% COMPLEX-VALUE OBJECT %%%%%%%%%%%%%%%%%%%%%%%%%%%
%  y_phi = checkerboard(64,2,2); y_abs=im2double(imread('Cameraman256.png'))+.2;
%  y_phi=im2double(imread('Cameraman256.png')); y_abs =checkerboard(64,2,2)+.2;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% DOUBLE-SIZE OBJECT DISTRIBUTION %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[yN,xN]=size(y_abs);
xNN=2*xN; yNN=2*yN;                  %% Computation window size 

temp=zeros(yNN,xNN); temp(yNN/4+1:yNN*3/4,xNN/4+1:xNN*3/4)=y_abs;
y_abs_double=temp;

temp=zeros(yNN,xNN); 
temp(yNN/4+1:yNN*3/4,xNN/4+1:xNN*3/4)=y_phi;
y_phi_double=temp;

u0_double=y_abs_double.*exp(j*(y_phi_double-1/2)*(pi/5));
 u0=u0_double(yNN/4+1:yNN*3/4,xNN/4+1:xNN*3/4);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%% PARAMETERS OF THE ALGORITHM %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


mu=1; 
gamma=100;       %% Regularization parameter for RI (initialization) 
It_Numb=200;     %% Number of Iterations
                 %% modulation_type=1 (complex),  modulation_type=2, (phase), modulation_type=3 (amplitude),
%% SELECT THE PARAMETERS OF LAGORITHM        
                 %% BASIC PARAMETERS for noiseless data
  threshold_ampl=.02; threshold_phi=.02; sigma=0.0; gamma_0=1; 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

                 %% BASIC PARAMETERS  for noisy data (Phase object,cameraman)
                   
% threshold_ampl=.02; threshold_phi=.02; sigma=0.02; gamma_0=1/5;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                 %% BM3D-frame PARAMETERS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Nstep = 2;  N11=4;  N22=8;

threshType='h';  %% 'h' for hard-thresholding; 's' for soft-thresholding
                 %%% hard-thresholding is recommended
                 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                 %% other parameters
smooth_index=1;   %  smooth_index=1 switches on smoothing in F-DDT forward propagation;
                  %  smooth_index=0 switches of filtering in F-DDT forwardpropagation

filtering=1;      % filtering=1 switches on BM3D filtering; filtering=0 switches of filtering
modulation_type=1; %% modulation type if the algorithm, modulation_type=1 for complex-valued object,
                   %%% modulation_type=2 for phase object, modulation_type=3 for module object

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                %% Scenario parameters


lambda1=.532*10^(-6);           %% Wave Length


delta_x=6.7*10^(-6);            %% pixel size

d=1.; % 2.0;                    %% Distance between the planes with respect to the focus distance, d=1 is in-focus

z_focus=delta_x^2*yN/lambda1;   %% focus distance

z=d*z_focus;                    %% distance between planes


 %% Forward propagation F-DDT (double size)
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[kernel]=Raylegh_Sommerfeld_Kernel_lambda_doublesmooth(z,lambda1, delta_x,yNN,xNN,smooth_index);

V=fft2(circshift(kernel,[-yNN/2,-xNN/2])); %% centered at the point (1,1)

U0_double=fft2(u0_double); 

U_Z_double=V.*U0_double;  

u_z_double_temp=ifft2(U_Z_double);  %% DOUBLE-SIZE COMPLEX VALUED NOISELESS OBSERVAIONS AT IMAGE PLANE


% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Noisy observations  %%%%%%%%%%%%%%%%%%%%%%%
randn('seed',1001);
u_z_double=zeros(yNN,xNN); noise=sigma*(randn(size(u_z_double))+j*randn(size(u_z_double)));

%% ZERO PADDED NOISY OBSERVATIONS
u_z_double(yNN/4+1:yNN*3/4,xNN/4+1:xNN*3/4)=u_z_double_temp(yNN/4+1:yNN*3/4,xNN/4+1:xNN*3/4)+noise(yNN/4+1:yNN*3/4,xNN/4+1:xNN*3/4);

%% SNGLE SIZE NOISY OBSERVATIONS (SENSOE DATA)
u_z_fixed=u_z_double(yNN/4+1:yNN*3/4,xNN/4+1:xNN*3/4); %% fixed observation of the single size

%% NON-ZERO PADDED NOISY OBSERVATIONS
u_z_double1=padarray(u_z_fixed,[yNN/4 xNN/4], 'replicate','both');


v0tmp=zeros(yNN/2,xNN/2);        % v_0 splitting varibale for filtering

u0_double_est=zeros(yNN,xNN);

ss=0;
%% CONVENTIONAL BACKWARD PROPAGATION (ASD) %%%%%%%%%%%%%%%%%
if ss==0
S = function_TransferFunction_ASD(z,lambda1,delta_x,xN,yN);
u0_ASD=ifft2(fft2(u_z_fixed).*conj(S));
ee_phi=angle(u0_ASD)-angle(u0);
    ee_abs=abs(u0_ASD)-abs(u0); 

rmse_abs_RI=sqrt(mean(ee_abs(:).^2));
rmse_phi_RI=sqrt(mean((ee_phi(:)).^2));

    figure(10), subplot(2,2,1),imshow(abs(u0_ASD),[]), title(['ASD, AMPL, RMSE = ' num2str(rmse_abs_RI,3)]), 
    subplot(2,2,2),imshow(angle(u0_ASD),[]), title(['ASD, PHASE, RMSE = ' num2str(rmse_phi_RI,3)]), ...
  
   subplot(2,2,3),plot(abs(u0(86,:)),'r'), hold on, plot(abs(u0_ASD(86,:))), V_aml=axis; 
   subplot(2,2,4),plot(angle(u0(86,:)),'r'), hold on, plot(angle(u0_ASD(86,:))), V_phi=axis;
end

%% Main iterations of the SPAR algorithm %%%%%%%%%%%%%%%%%%%


for ss=[1:It_Numb]; 
    ss;

temp=zeros(yNN,xNN); temp(yNN/4+1:yNN*3/4,xNN/4+1:xNN*3/4)=v0tmp;
v0tmp_double=temp;

%% INVERSE ESTIMATION %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if ss==1 %%% INITIALIZATION, first iteration, regularized inverse

u0_double_est=ifft2((fft2(u_z_double1).*conj(V)/mu+1/gamma)./(abs(V).^2/mu+1/gamma));  

else   %%% NORMAL ITERATION %%%%%%%%%%%%%%%%%%%%%%

u0_double_est=ifft2((fft2(u_z_double).*conj(V)/mu+fft2(v0tmp_double)/gamma_0)./(abs(V).^2/mu+1/gamma_0));  
end

%%%%%%%%%%%%%%%%%%%%%%% FILTERING %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
u0temp=u0_double_est(yNN/4+1:yNN*3/4,xNN/4+1:xNN*3/4); %% SMALL ESTIMATE
u0_est=u0_double_est(yNN/4+1:yNN*3/4,xNN/4+1:xNN*3/4);

%%%%%%%%%%%%%%%%%%%%%%% Imaging of the first iteration %%%%%%%%%%%%%\
if 0
if ss==1
    ee_phi=angle(u0_est)-angle(u0);
    ee_abs=abs(u0_est)-abs(u0); 

rmse_abs_RI=sqrt(mean(ee_abs(:).^2));
rmse_phi_RI=sqrt(mean((ee_phi(:)).^2));
u0_est1=u0_est; 
    figure(33), subplot(2,2,1),imshow(abs(u0_est),[]), title(['RI, AMPL, RMSE = ' num2str(rmse_abs_RI,3)]), 
    subplot(2,2,2),imshow(angle(u0_est1),[]), title(['RI, PHASE, RMSE = ' num2str(rmse_phi_RI,3)]), ...
   % subplot(2,2,3),plot(abs(u0(86,:)),'r'), hold on, plot(abs(u0_est1(86,:))), V_aml=axis; subplot(2,2,4),plot(angle(u0(86,:)),'r'), hold on, plot(angle(u0_est1(86,:))), V_phi=axis;
   subplot(2,2,3),plot(abs(u0(86,:)),'r'), hold on, plot(abs(u0_est1(86,:))), V_aml=axis; 
   subplot(2,2,4),plot(angle(u0(86,:)),'r'), hold on, plot(angle(u0_est1(86,:))), V_phi=axis;

RMSE_RI=[rmse_abs_RI rmse_phi_RI]
    
end
end

%% optional, padding for BM3D filtering
  kk=30;  u0_est_filt=padarray(u0_est, [kk kk],'replicate', 'both'); %% can be k=0;

%% BM3D-frame filtering
    [v0tmp1] = SPAR_BM3D(u0_est_filt, threshold_phi, threshType, threshold_ampl, N11, N22, Nstep,modulation_type,filtering,ss);
    v0tmp=v0tmp1(kk+1:end-kk,kk+1:end-kk);
  
 % u0_est=v0tmp; %% OPTIONAL 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
 
%% DOUBLE-SIZE F-DDT FORWARD PROPAGATION
temp=zeros(yNN,xNN); temp(yNN/4+1:yNN*3/4,xNN/4+1:xNN*3/4)=u0_est;

u0_double_est=temp; % zeroed bounds

U0_double_est=fft2(u0_double_est); 

u_z_double=ifft2(V.*U0_double_est);

u_z_double(yNN/4+1:yNN*3/4,xNN/4+1:xNN*3/4)=u_z_fixed;  % insertion of the fixed observation in the central part of the image plane



%% RMSE CALCULATION %%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ee_abs=abs(u0_est)-abs(u0); 
ee_phi=angle(u0_est)-angle(u0);
rmse_abs(ss)=sqrt(mean(ee_abs(:).^2));
rmse_phi(ss)=sqrt(mean(ee_phi(:).^2));
%% RMSE iteration imaging 
figure(1), subplot(1,2,1),plot(rmse_abs), title('RMSE, AMPLITUDE'), grid on, hold on, plot(rmse_abs,'o'),...
    xlabel('ITERATION NUMBER'), subplot(1,2,2),plot(rmse_phi),grid on, hold on, plot(rmse_phi,'o'), title('RMSE, PHASE'), grid on,....
    xlabel('ITERATION NUMBER')

%% RECONSTRACTION IMAGING, ITERATIONS
if 0
figure(30), subplot(2,2,1),imshow(abs(u0_est),[]), title(['SPAD, AMPL, RMSE = ' num2str(rmse_abs(end),3)]), 
subplot(2,2,2),imshow(angle(u0_est),[]), title(['SPAR, PHASE, RMSE= ' num2str(rmse_phi(end),3)]), ...

subplot(2,2,3),  plot(abs(u0_est(86,:))), axis(V_aml), subplot(2,2,4), plot(angle(u0_est(86,:))), axis(V_phi);


end
RMSE=[It_Numb gamma gamma_0 Nstep ss rmse_abs(ss) rmse_phi(ss)]
end

%% IMAGING OF RECONSTRACTION RESULTS
figure(3), subplot(2,2,1),imshow(abs(u0_est),[]), title(['SPAR, AMPL, RMSE = ' num2str(rmse_abs(end),3)]), 
subplot(2,2,2),imshow(angle(u0_est),[]), title(['SPAR, PHASE, RMSE= ' num2str(rmse_phi(end),3)]), ...
subplot(2,2,3), plot(abs(u0(86,:)),'r'), hold on, plot(abs(u0_est(86,:))), axis(V_aml), subplot(2,2,4),
plot(angle(u0(86,:)),'r'), hold on, plot(angle(u0_est(86,:))), axis(V_phi);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

toc
% figure(1), subplot(1,2,1),plot([1:3:200],rmse_abs(1:3:200)), title('RMSE, AMPLITUDE'), grid on, hold on, plot([1:3:200],rmse_abs(1:3:200),'o'),...
%     xlabel('ITERATION NUMBER'), subplot(1,2,2),plot([1:3:200],rmse_phi(1:3:200)),grid on, hold on, plot([1:3:200],rmse_phi(1:3:200),'o'), title('RMSE, PHASE'), grid on,....
%     xlabel('ITERATION NUMBER')
