function S = function_TransferFunction_ASD(z,lambda,delta,N,M)
% function_TransferFunction_ASD() calculates the transfer function of the
% angular spectrum deconvolution (ASD) as following [1]:
% ASD[fx,fy]=exp((j*2*pi*z/lambda)*sqrt(1-lambda^2*((fx)^2+(fy)^2)));
% Notation:
% S = function_ASD(z,lambda,delta,N,M)
% S         the output ASD transfer function of the size N x M
% The input setting variables:
% z         a distance between the input and output planes, [m]
% lambda    a wavelength, [m]
% delta     a pixel size, the same as for the Fourier image of a wave
%           field distribution (e.g. u0 in the following example), [m]
% N x M     a size of the kernel (# of rows x # of columns), [# of pixels]
% 
% Example:
% let us compute the wave field distribution at the sensor plane uz of the 
% size equal to the size of the object distribution u0 (NxM=512x512)
% 1. calculation of the ASD transfer function for some parameters
% >> S = function_TransferFunction_ASD(0.044,6328e-10,7.4e-6,512,512);
% 2. convolution in the frequency domain
% >> uz = ifft2(fft2(u0).*S);
% References:
% [1]  J. W. Goodman, Introduction to Fourier Optics, New York: McGraw-Hill
% Inc, Second Edition, 1996. 
% 2010 (c) Artem Migukin, Vladimir Katkovnik, v. 1.0.22.9.10
S = function_TransferFunction_ASD(z,lambda,delta,N,M);